#include "PHGeometry.h"
#include <AppKit/NSAffineTransform.h>
#include <math.h>

//#define MIN(x, y) (((x) < (y)) ? (x) : (y))
//#define MAX(x, y) (((x) < (y)) ? (y) : (x))
#define DISTANCE(x, y) ( ((x) < (y)) ? ((y) - (x)) : ((x) - (y)) )
#define SIGN(x) ((x) > 0 ? 1 : -1)

NSPoint PHMovePoint(NSPoint pt, float dx, float dy)
{
    pt.x += dx;
    pt.y += dy;
    return pt;
}

NSPoint PHIntegralPoint(NSPoint pt)
{
    pt.x = floorf(pt.x);
    pt.y = floorf(pt.y);
    return pt;
}

NSPoint PHMoveOrigin(NSPoint pt, NSPoint delta_origin)
{
    pt.x -= delta_origin.x;
    pt.y -= delta_origin.y;
    return pt;
}

NSRect PHRectFromPoints(NSPoint a, NSPoint b)
{
    return NSMakeRect(MIN(a.x, b.x),
                      MIN(a.y, b.y),
                      DISTANCE(a.x, b.x),
                      DISTANCE(a.y, b.y));
}

float PHDistance(NSPoint a, NSPoint b)
{
    //return MAX( DISTANCE(a.x, b.x), DISTANCE(a.y, b.y) );    
    return MIN( DISTANCE(a.x, b.x), DISTANCE(a.y, b.y) );    
}

NSPoint PHClosestPointInRect(NSPoint pt, NSRect rc)
{
    float x, y;
    if (pt.x < NSMinX(rc))
        x = NSMinX(rc);
    else if (pt.x > NSMaxX(rc))
        x = NSMaxX(rc);
    else
        x = pt.x;

    if (pt.y < NSMinY(rc))
        y = NSMinY(rc);
    else if (pt.y > NSMaxY(rc))
        y = NSMaxY(rc);
    else
        y = pt.y;

    return NSMakePoint(x, y);
}

NSPoint PHClosestPointOnRectEdge(NSPoint pt, NSRect rc)
{
    if (NSPointInRect(pt, rc) == NO)
        return PHClosestPointInRect(pt, rc);

    // find closest edge
    
    float d[4];
    d[0] = pt.x - NSMinX(rc);
    d[1] = pt.y - NSMinY(rc);
    d[2] = NSMaxX(rc) - pt.x;
    d[3] = NSMaxY(rc) - pt.y;

    // find minimum in the array d[]
    int i_min = 0;
    float d_min = d[i_min];
    int i;
    for (i=1; i < 4; ++i)
        if (d[i] < d_min)
            d_min = d[ i_min = i ];

    NSPoint result;
    switch (i_min)
    {
        case 0: result = NSMakePoint(NSMinX(rc), pt.y); break;
        case 1: result = NSMakePoint(pt.x, NSMinY(rc)); break;
        case 2: result = NSMakePoint(NSMaxX(rc), pt.y); break;
        case 3: result = NSMakePoint(pt.x, NSMaxY(rc)); break;
    }
    return result;
}

NSRect PHPushRectInsideBoundaries(NSRect aRect, NSRect bound)
{
    NSRect result = aRect;
    if (aRect.size.width <= bound.size.width &&
        aRect.size.height <= bound.size.height)
    {
        if (result.origin.x < bound.origin.x)
            result.origin.x = bound.origin.x;
        else if (result.origin.x + result.size.width >
                 bound.origin.x + bound.size.width)
            result.origin.x =
                bound.origin.x + bound.size.width - result.size.width;

        if (result.origin.y < bound.origin.y)
            result.origin.y = bound.origin.y;
        else if (result.origin.y + result.size.height >
                 bound.origin.x + bound.size.height)
            result.origin.y =
                bound.origin.y + bound.size.height - result.size.height;
    }
    return result;
}

NSAffineTransform * PHReverseTransform(NSAffineTransform * xform)
{
    NSAffineTransform * reverse = [xform copy];
    [reverse invert];

    return AUTORELEASE(reverse);
}

Vect2D PHBoundVect2D(NSPoint start, Vect2D v, NSRect rc)
{
    // Check that start point is in rect
    
    // Move referrence frame origin to the origin of rc
    //start.x -= rc.origin.x;
    //start.y -= rc.origin.y;

    float shape = fabs(v.dy / v.dx);
    float dx1 = MIN(fabsf(v.dx),
                    (v.dx > 0) ? rc.size.width - start.x + rc.origin.x :
                    start.x - rc.origin.x);

    float dy1 = dx1 * shape;


    float dy2 = MIN(fabsf(v.dy),
                    (v.dy > 0) ? rc.size.height - start.y + rc.origin.y :
                    start.y - rc.origin.y);

    float dx2 = (shape != 0) ? dy2 / shape : 0;

    // take minimal size

    float dx = SIGN(v.dx) * MIN(dx1, dx2);
    float dy = SIGN(v.dy) * MIN(dy1, dy2);

    return PHMakeVect2D(dx, dy);
}

NSString * PHStringFromInterval(Interval intvl)
{
    return [NSString stringWithFormat:
                         @"{min=%g max=%g}", intvl.min, intvl.max];
}
